package edu.wpi.cs.dsrg.utils;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.io.*;

/**
 * This class is Used to do a file copy.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */
 
public class FileCopy {
/**
 * Copy source file into the destination file.
 * @param fileinput Input file name.
 * @param fileoutput Output file name.
 * @param overwrite True to overwrite the outputfile, otherwise don't overwrite.
 * @since 1.0
 */
public static void copy(
    String fileinput,
    String fileoutput,
    boolean overwrite) {
    BufferedInputStream in = null;
    BufferedOutputStream out = null;

    try {

        try {
            in = new BufferedInputStream(new FileInputStream(fileinput));

            File fout = new File(fileoutput);

            if (fout.exists()) {
                if (!overwrite) {
                    System.out.println(
                        "File " + fileoutput + " already exist! Please remove it and try again!");
                    return;
                } else { // ger rid of this file.
                    System.out.println(
                        "File " + fileoutput + " already exist! It's going to be overwritten!");
                    fout.delete();
                }
            }

            out = new BufferedOutputStream(new FileOutputStream(fileoutput));
            byte buffer[] = new byte[1000];

            int count = -1;

            do {
                count = in.read(buffer, 0, 1000);
                if (count == -1) // if end of file, break the loop.
                    break;
                out.write(buffer, 0, count);
            } while (count != -1);

            System.out.println(fileinput + " --> " + fileoutput + " is done!");

        } catch (FileNotFoundException e) {
            System.out.println(e.getMessage());
        } finally {
            if (in != null)
                in.close();
            if (out != null)
                out.close();
        }
    } catch (IOException ie) {
        System.out.println(ie.getMessage());
    }
}
public static void main(String args[]) {
    copy(args[0], args[1], true);
}
}
