package edu.wpi.cs.dsrg.xmldb.xat.common.operator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;

/**
 * An Interface to represent a columns Name List
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0      
 */

public interface ColumnNameList extends Visitable {
    /**
     * Appends a new column name to the end of column names list.
     * @param newCol is column name.
     * @since   1.0          
     */
    void addColumnName(String newCol);
    /** 
      * Deletes a column name from the list
      * @param oldName is column name
      * @since   1.0           
      */
    void deleteColumnName(String oldName);
    /**
     * Returns a list of column names.
     * @return a list column names.
     * @since   1.0          
     */

    String[] getColumnNames();
    /** 
      * Inserts a column name in the list. 
      * @param newName is column name.
      * @param position is insertion poistion.
      * @since   1.0           
      */
    void insertColumnName(String newName, int position);
    /** 
      * Sets a column name in the certain position in the list. 
      * @param newName is column name.
      * @param position is insertion poistion.
      * @since   1.0           
      */
    void setColumnName(String newName, int position);
    /** 
      * Sets a list of column names to be projected out.. 
      * @param newName is column name.
      * @param position is insertion poistion.
      * @since   1.0           
      */

    void setColumnNames(String[] columns);
    /**
     * Updates a specific column name in the list with a new name.
     * @param oldname is the old column name.
     * @param newname is new column name.
     * @since   1.0     
     */
    void updateColumnName(String oldname, String newname);
}