package edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

/**
 * Implementation for the interface NameColumn.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0      
 */

public class NameColumnImp extends XATOperatorImp implements NameColumn {
    /**
    * the name of the new column
    */
    protected String sNewColumnName;
    /**
    * the original name of the column
    */
    protected String sOrgColumnName;
    /**
    * the absolute path of the column to be renamed
    */
    protected NavigationStep[] columnPath;

    /**
    * Block description.
    * used to extend the functionality of namecolumn operator.
    */
    protected String desc = null;

    /**
     * Assigns a node to this operator .
     * @param init edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0          
     */
    public NameColumnImp(XATNode xnodInit) {
        super(xnodInit);
    }
    /**
     * Assigns a node to this operator also assigns original and new column names.
     * @param init edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param sOrgNameInit original column name
     * @param sNewNameInit new column name
     * @since   1.0          
     */
    public NameColumnImp(
        XATNode xnodInit,
        String sOrgNameInit,
        String sNewNameInit) {
        super(xnodInit);
        sOrgColumnName = sOrgNameInit;
        sNewColumnName = sNewNameInit;
    }
    /**
     * Assigns a node to this operator also assigns original and new column names and column path.
     * @param init edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param sOrgNameInit original column name
     * @param sNewNameInit new column name
     * @param colPath is column path
     * @since   1.0          
     */
    public NameColumnImp(
        XATNode xnodInit,
        String sOrgNameInit,
        String sNewNameInit,
        NavigationStep[] colPath) {
        super(xnodInit);
        sOrgColumnName = sOrgNameInit;
        sNewColumnName = sNewNameInit;
        columnPath = colPath;
    }
    /** 
      * It used to implement a visitor pattern. 
      * @param visitor for the visitor pattern
      * @since   1.0     
      */
    public void accept(Visitor visitor) {
    }
    /**
     * Extends the semantic of NameColumn operator.
     * It also used to identify a start of a new query block.
     * This will give back query block description.
     * So far, we have one meaningful desc: "return".
     * Which is used to tell the decore engine to put the LOJ
     * below that operator.
     * @return java.lang.String
     * @since   1.0          
     */
    public java.lang.String getBlockDesc() {
        return this.desc;
    }
    /**
     * Gets the absolute path for this column.
     * @return NavigationStepImp[]
     * @since   1.0          
     */
    public NavigationStep[] getColumnPath() {
        return columnPath;
    }
    /**
     * Gets the new column name.
     * @return new column name
     * @since   1.0          
     */
    public String getNewColumnName() {
        return sNewColumnName;
    }
    /**
     * Gets the original column name.
     * @return original column name
     * @since   1.0          
     */

    public String getOriginalColumnName() {
        return sOrgColumnName;
    }
    /**
     * Sets block description.
     * @param desc java.lang.String
     * @see getBlockDesc()
     * @since   1.0          
     */
    public void setBlockDesc(java.lang.String desc) {
        this.desc = desc;
    }
    /**
     * Sets the absolute path for this rename.
     * @param newColumnPath NavigationStepImp[]
     * @since   1.0          
     */
    public void setColumnPath(NavigationStep[] newColumnPath) {
        columnPath = newColumnPath;
    }
    /**
     * Sets the new name for the column
     * @param new_name new name for the column
     * @since   1.0         
     */
    public void setNewColumnName(String new_name) {
        sNewColumnName = new_name;
    }
    /**
      * Set the name of the column to be renamed
      * @param original_name name of the column to be renamed
      * @since   1.0          
      */
    public void setOriginalColumnName(String original_name) {

        sOrgColumnName = original_name;

    }
    /**
      * returns a string representation 
      * @return java.lang.String
      * @since   1.0         
      */
    public String toString() {
        return "NameColumn: Old - "
            + getOriginalColumnName()
            + ", New - "
            + getNewColumnName();
    }
}
