package edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator;
/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

/**
 * Implementation for the Navigate interface. The Navigate opertor takes the source and navigates through it given a specific XPath
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0   
 */
public abstract class NavigateImp extends XATOperatorImp implements Navigate {

    //entry point of the navigate node
    protected NavigationStep[] entry;

    //destination of the navigate node
    protected NavigationStep[] dest;

    /**
     * Assigns a node to this operator.
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0   
     */
    public NavigateImp(XATNode xnodInit) {
        super(xnodInit);

    }

    /**
     * Assigns a node to this operator, an entry point, and a destination .
     * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @param ep edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavigationStep[]
     * @param d edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavigationStep[]
     * @since   1.0   
     */
    public NavigateImp(XATNode node, NavigationStep[] ep, NavigationStep[] d) {
        super(node);
        entry = ep;
        dest = d;

    }

    /** 
     * Gets the binding value.
     * @return String
     * @since   1.0  
     * @deprecated Use getOutputColumnName() instead.     
     */
    public String getBinding() {
        return getOutputColumnName();
    }

    /** 
     * Gets the destination.
     * @return an array of NavigationStep
     * @since   1.0        
     */
    public NavigationStep[] getDestination() {
        return dest;
    }

    /** 
     * Gets the entry point.
     * @return an array of NavigationStep
     * @since   1.0        
     */
    public NavigationStep[] getEntryPoint() {
        return entry;
    }

    /**
     * Returns a String that represents the value of this object.
     * @return a String representation of the object
     * @since   1.0          
     */
    public String printNode() {
        StringBuffer sb = new StringBuffer();
        sb.append("Navigate Node: \n  entry: ");
        for (int i = 0; i < entry.length; i++)
            sb.append(entry[i].getStepValue() + " ");
        sb.append("\n destination: ");
        for (int j = 0; j < dest.length; j++)
            sb.append(dest[j].getStepValue() + " ");
        sb.append("\n binding: " + getOutputColumnName());
        return sb.toString();
    }

    /** 
     * Sets the binding value.
     * @param bind String
     * @since   1.0
     * @deprecated Use setOutputColumnName(String) instead.     
     */
    public void setBinding(String bind) {
        setOutputColumnName(bind);
    }

    /** 
     * Sets the destination.
     * @param destination is an array of NavigationStep
     * @since   1.0
     */
    public void setDestination(NavigationStep[] destination) {
        dest = destination;
    }

    /**
     * Sets the entry point. 
     * @param ept New entry point for this navigation operator.
     * @since   1.0     
     */
    public void setEntryPoint(NavigationStep[] ept) {
        entry = ept;
    }

    /**
     * Abstract method.
     * @return a String
     * @since   1.0          
     */
    public abstract String toString();

    /**
     * Returns a String that represents a navigation path in XPath format.
     * @return a String
     * @since   1.0          
     */
    String toXPath(NavigationStep[] steps) {

        String result = "";
        if (steps == null)
            return result;
        for (int i = 0; i < steps.length; i++)
            result += "/" + steps[i];
        return result;
    }
}