package edu.wpi.cs.dsrg.xmldb.xat.common.treestructure;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.dompattern.*;

/**
 * This Class defines the DOMpattern data structure. 
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public interface DOMPattern extends Pattern {
    /**
     * Add a new parent node for a given DOMPatternNode node. 
     * The parent node is updated to have a link to the child node.
     * The child node is updates to have a link to the parent node.
     * @return void
     * @param parent The parent node it attached to.
     * @param child The child node attached.
     * @since 1.0
     */
    public void addNode(DOMPatternNode child, DOMPatternNode newparent);
    /** 
     * Get the root node of this DOMTree.
     * @since 1.0
     */
    public RootNode getRoot();
    /**
     * Check the tree for a specific node
     * @return boolean True: The given DOMPatternNode is in the tree. False: Otherwise.
     * @param node the node you are looking for in the tree
     * @since 1.0
     */
    public boolean isInTree(DOMPatternNode node);
    /**
     * Delete the given DOMPatternNode. 
     * It will affect number of nodes. All the children nodes of the
     * to-be-deleted node will be the children of the parent node of the to-be-delete node.
     * @return void
     * @param node the to-be-deleted child node.
     * @since 1.0
     */
    public void removeNode(DOMPatternNode node);
    /**
     * Set root node of DOMPattern.
     * @return void.
     * @param new_root The new root node.
     * @since 1.0
     */
    public void setRoot(RootNode new_root);
    /**
     * Get the indented print output of the result.
     * This method is used for the GUI.
     * @return String The indented print output of the result.
     * @since 1.0     
     */
    public String toIndentString();
    /**
     * Convert the tree into a list by calling toString() on each node.
     * @return ListPattern The result listPattern of given DOMPattern Node.
     * @param node The DOMPatternNode translate for.
     * @since 1.0     
     */
    public ListPattern toListPattern(DOMPatternNode node);
    /**
     * Convert the tree into a String.
     * @return String
     * @since 1.0
     */
    public String toString();
}
