package edu.wpi.cs.dsrg.xmldb.xat.common.treestructure;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;

/**
 * This Class implemented the DOMpattern Interface.
 * This pattern is good for execution. It will be a list of mix of strings and column names.
 * For example: (seperated by |, [] means a column name.) |<tag attr="|[col1]|">|[col2]|Foo|[col3]|</tag>|
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public interface ListPattern extends Pattern {

    /**
     * Add the given column name into a ListPattern.
     * @return void
     * @param param The ColumnName adding to.
     * @since 1.0
     */
    public void add(ColumnName param);
    /**
     * Connect with another list pattern.
     * @return void     
     * @param text java.lang.String
     * @since 1.0     
     */
    public void add(ListPattern pattern);
    /**
     * Add text in list pattern.
     * Creation date: (2/9/2003 10:26:56 PM)
     * @param text java.lang.String
     * @since 1.0     
     */
    public void add(String text);
    /**
     * Get the list.
     * @return Vector The vector of the list pattern.
     * @since 1.0     
     */    
    public Vector getList();
    /**
     * Set the list in the ListPattern.
     * @return void
     * @param newPattern  
     * @since 1.0     
     */        
    public void setList(Vector newPattern);
    /**
     * Convert the ListPattern into String.
     * @return String The result of converting.
     * @since 1.0     
     */            
    public String toString();
}