package edu.wpi.cs.dsrg.xmldb.xat.common.xatnode;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;

/**
 * This class is the column structure of the XAT. 
 * It's a table of XML fragments. 
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0 
 */

public interface ColumnAccessor{
    /**
     * Append an empty column to the end of the table.
     * @return int The position the new appended column.
     * @param new_column The new column append will be appended.
     * @since 1.0
     */
    int appendColumn(String sColumnName) throws XATTableTupleNoMatchException;
    /**
     * Append a new column to the end of the table.
     * @return int The position the new appended column.
     * @param new_column The new columns.
     * @since 1.0     
     */
    int appendColumn(String sColumnName, Vector new_column)
        throws XATTableTupleNoMatchException;
    /**
     * Delete a column in the specific position.
     * @return Vector The deleted column.
     * @param position The position of the column will be deleted.
     * @since 1.0     
     */
    Vector deleteColumn(int position)
        throws XATTableOutOfRangeException, XATTableEmptyException;
    /**
     * Get the vector containing the columns themselves.
     * @return Vector The vector containing the contents.
     * @since 1.0     
     */
    Vector getColList();
    /**
     * Get the vector containing the column names.
     * @return Vector The vector containing the names.
     * @since 1.0     
     */
    Vector getColNames();
    /**
     * Get a column back by position.
     * @return Vector The vector of cells in one column.
     * @param position The position of the column which starts from 1.
     * @since 1.0     
     */
    Vector getColumn(int position) throws XATTableOutOfRangeException;
    /**
     * Get the column name for a specific column position.
     * @return Vector The column names for a give position.
     * @param position The column position searching for.
     * @since 1.0     
     */
    String getColumnName(int position) throws XATTableOutOfRangeException;
    /**
     * Get the Column Position from the Column Name.
     * @return int The position of the column.
     * @param column_name The column name.
     * @since 1.0     
     */
    int getColumnPosition(String column_name)
        throws XATTableColumnNoExistException;
    /** 
     * Get total number of columns in this table.
     * @return int The number of columns in the table.
     * @since 1.0     
     */
    int getNumberOfColumns();
    /**
     * Insert a new column to the table before specific position.
     * @return int The number of Columns after the insertion.
     * @param position The column position.
     * @param sColumnName The new column name.
     * @param new_column The new column content.
     * @since 1.0     
     */
    int insertColumn(int position, String sColumnName, Vector new_column)
        throws XATTableOutOfRangeException, XATTableTupleNoMatchException;
    /**
     * This method takes two XATTables (this and tab2).  Finds all matching columns, adds those to the head of a vector,
     *  and appends all other column names at the end.  Appends tab1 column names, then tab2
     * @return The result Vector containning all the column names of both this and given tables.
     * @param tab2 The XATTable merged to.
     * @since 1.0    
     */
    public Vector mergeColumnNames(XATTable tab2);
    /**
     * Move a column to a specific position.
     * @return void
     * @param from_position The position of the column which is going to moved.
     * @param to_position The position of the column will be moved to.
     * @since 1.0     
     */
    void moveColumn(int from_position, int to_position)
        throws XATTableOutOfRangeException, XATTableEmptyException;
    /**
     * Update the column in given position as new column.
     * @return Vector The original column.
     * @param position The position of the column will be updated.
     * @param new_column The new column.
     * @since 1.0     
     */
    Vector updateColumn(int position, Vector new_column)
        throws
            XATTableOutOfRangeException,
            XATTableEmptyException,
            XATTableTupleNoMatchException;
}
