package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Hashtable;
import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;

/**
 * This Class is used to cancel out the reduncent operators 
 * by traversing the XAT.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

 public class CancelOutTraversal extends ReWriteTraversal{

/**
 * The hashtable of the operators which will be deleted .
 * @since 1.0
 */
 Hashtable cancelOutList;
	
/**
 * CancelOutTraversal consttructor.
 * Constructs a CancelOutTraversal object using XAT and Rewrite Rules and order.
 * @param tree  The XML Algebra Tree (XAT) going to be rewrited.
 * @param p1    The rewrite rule categories.
 * @param order The order of operator being considered in the rewrite rule categories.
 * @since 1.0   
 * @see   edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 * @see   edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.ReWriteRules
 */
public CancelOutTraversal(XATTree tree, ReWriteRules p1, Vector order) {
	super(tree,p1,order);
	cancelOutList = new Hashtable();	
}
/**
 * CancelOutTraversal consttructor.
 * Constructs a CancelOutTraversal object using XAT and Rewrite Rules and order.
 * @param tree  The XML Algebra Tree (XAT) going to be rewrited.
 * @param p1    The rewrite rule categories.
 * @param order The order of operator being considered in the rewrite rule categories.
 * @since 1.0   
 * @see   edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 * @see   edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.ReWriteRules
 */
public CancelOutTraversal(XATTree tree, ReWriteRules p1, Vector order, boolean step) {
	super(tree,p1,order, step);
	cancelOutList = new Hashtable();	
}
/**
 * CancelOut redundent operators.
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 * @since 1.0
 * @see   edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */

public XATTree doCancelOut() {
    Vector sourceSet = new Vector();
    String operatorToCancel;
    XATNode currentNode, nextAncestorNode;
    ReWriteResult result = new ReWriteResult();

    //make a copy of the order of operators to pushdown
    Vector cancelOutOrderCopy = new Vector(this.reWriteOrder);

    //find all of the source nodes for the mapping query	
    getAllSourceNodes(sourceSet, reWriteTree.getRoot(), reWriteTree);

    while (cancelOutOrderCopy.size() > 0) {
        //get the name of the operator to cancel out
        operatorToCancel = (String) cancelOutOrderCopy.elementAt(0);
        if (isDebug()) {
            System.out.println("Attempt to cancel " + operatorToCancel);
        }
        currentNode = new XATNodeImp();
        nextAncestorNode = new XATNodeImp();
        for (int x = 0; x < sourceSet.size(); x++) {
            //get the next source node found
            XATNode s1 = (XATNode) sourceSet.elementAt(x);
            //find the next node that has not been attempted for cancel out
            currentNode = findNextNode(s1, operatorToCancel);
            while (currentNode != null) {
                nextAncestorNode = findNextNode(currentNode, operatorToCancel);
                if (nextAncestorNode != null) {
                    do {
                        //evalute the equivalence rules and cancel out if possible
                        try {
                            result = (ReWriteResult) rules.evaluateRules(currentNode, nextAncestorNode);
                        } catch (Exception e) {
                            System.out.println(e.toString());
                            //no rules found
                            result.setFinalPosition(true);
                        }
                    } while (
                        (nextAncestorNode = findNextNode(nextAncestorNode, operatorToCancel)) != null);

                }
                currentNode = findNextNode(currentNode, operatorToCancel);
            }
        }
        if (isStepThrough()) {
            stepTrees.addElement(((XATTree) reWriteTree).clone());
        }
        cancelOutOrderCopy.removeElementAt(0);
    }
    return reWriteTree;
}
}
