package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dompattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

import java.lang.reflect.*;
import java.util.*;

/**
 * This Class is used to merge operators by traversing the XAT.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class MergeOperatorRules extends ReWriteRules {
/**
 * Default MergeOperatorRules constructor.
 * Constructs a MergeOperatorRules object.
 * @since 1.0   
 */
public MergeOperatorRules() {
    super();
}
/**
 * Find if the two taggers are mergable or not.
 * @return void
 * @param  parent		The parent Tagger node.
 * @param  child		The child Tagger node.
 * @param  pPatternNode The DOMPatternNode of parent Tagger.
 * @param  mergable 	The Vector of mergable element.
 * @since  1.0   
 * @see    DOMPatternNode
 * @see    Tagger
 */

protected void findMergable(
    Tagger parent,
    Tagger child,
    DOMPatternNode pPatternNode,
    Vector mergable) {

    XATTable xtabInput;
    XATTable xtabOutput = null;
    String cColName = child.getOutputColumnName();
    DOMPatternNode[] cList = pPatternNode.getChildren();

    for (int i = 0; i < cList.length; i++) {
        DOMPatternNode el = cList[i];
        if (el instanceof TagNode) {
            findMergable(parent, child, el, mergable);
        } else {
            if (el instanceof ColumnNameNode) {
                String pColName = ((ColumnNameNode) el).getColumnName();
                if (pColName.equals(cColName) && (!isUsedAbove(parent.getNode(), cColName))) {
                    mergable.addElement(el);
                } else {
                }
            } else {
                // comments from Ling Wang: It shouldn't throw an exception.
                // Instead, it should just ignore it. because here we only
                // focus on the ColumnNameNode which can be inlined further.
                // throw new RuntimeException("invalid list element " + el.toString());
            }
        }
    }
}
/**
 * Merges two Navigates
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param  parent		The parent Navigate node.
 * @param  child		The child Navigate node.
 * @since  1.0   
 * @see    Navigate
 */
protected ReWriteResult mergeNavigates(Navigate parent, Navigate child) {
    if (isDebug()) {
        System.out.println("********* In merge Nav / Nav *************");
    }
    ReWriteResult p1 = new ReWriteResult();
    String childBinding = child.getBinding();
    String parentEntry =
        ((NavigationStep[]) (parent.getEntryPoint()))[0].getStepValue();
    //no need to look up values in the binding table, a simple string match is suffcient
    //and see if childBinding is used elsewhere
    if (parentEntry.equals(childBinding)
        && !isUsedAbove(parent.getNode(), childBinding)) {
        if (isDebug()) {
            System.out.println("$$$$$$$$$$$The 2 Navigates will be merged$$$$$$$$$$$");
            System.out.println(
                "$$$$$$$$$$$ "
                    + parent.toString()
                    + " is merged with "
                    + child.toString()
                    + " $$$$$$$$$$$");
        }
        /*
        * they can be merged so the parent Navigate will be modified
        * the child's output column will be change to that of the parent
        * the child's destination will have the destination of the parent concatted to it
        * then we delete the parent
        */
        try {
            NavigationStep[] parentDest = parent.getDestination();
            NavigationStep[] childDest = child.getDestination();
            NavigationStep[] combinedDest = combineSteps(childDest, parentDest);
            parent.setDestination(combinedDest);

        } catch (NullPointerException e) {
            parent.setDestination(child.getDestination());
        }
        parent.setEntryPoint(child.getEntryPoint());
        if (isDebug()) {
            System.out.println("$$$$$$$$$$$ The new parent is " + parent.toString());
        }

        //delete the parent node, but it will really delete the child b/c we swap them
        // Xin: Really? I changed from "true" to "false". Now the SQLStmt works. 9/17/2002.
        // Brian: instead, let's just delete the child, and not swap.  Simple.  4/7/2003
        p1.setDeleteChildNode(true);
        //swap(parent, child);
        //p1.setSwapped(true);
        p1.setSwapped(false);
        //make sure the parent is not in its final position
        p1.setFinalPosition(false);
    } else {
        p1.setSwapped(false);
        p1.setFinalPosition(true);
    }

    return p1;
}
/**
 * Merge two Taggers if possible.
 * @return int		The size of the mergable node in Tagger Pattern.
 * @param  parent	The parent Tagger node.
 * @param  child	The child Tagger node.
 * @since  1.0   
 * @see    Tagger
 * @see    findMergable( Tagger, Tagger, DOMPatternNode, Vector) findMergable
 */
protected int mergeTaggers(Tagger parent, Tagger child) {
    //Get the information from parent and child Tagger
    // Begin to searching for merge
    Vector mergable = new Vector();

    DOMPatternNode pPatternNode =
        (DOMPatternNode) ((DOMPattern) parent.getPattern()).getRoot();
    findMergable(parent, child, pPatternNode, mergable);

    int maxmergable = mergable.size();
    if (maxmergable > 0) {
        //merge two pattern
        DOMPatternNode[] childrenList =
            ((DOMPattern) child.getPattern()).getRoot().getChildren();
        for (int i = 0; i < maxmergable; i++) {
            DOMPatternNode currentNode = (DOMPatternNode) mergable.elementAt(i);
            DOMPatternNode pnode = currentNode.getParent();
            int position = pnode.deleteChild(currentNode);
            for (int j = 0; j < childrenList.length; j++) {
                pnode.addChild(childrenList[j], position);
                childrenList[j].setParent(pnode);
                position++;
            }
        }
    }
    return maxmergable;
}
/**
 * Rule to merge GroupBy with GroupBy
 * @return void
 * @param  parent	The parent GroupBy node.
 * @since  1.0   
 * @see    GroupBy
 */
protected void reduceGroupBy(GroupBy parent) {
    if (isDebug()) {
        System.out.println("********* In reduce GroupBy/GroupBy *************");
    }

    XATTree attachedTree = parent.getXATTree();
    if (attachedTree.getRoot().getOperator() instanceof GroupBy) {
        GroupBy child = (GroupBy) attachedTree.getRoot().getOperator();
        // recursively reduce the children.
        reduceGroupBy(child);

        // then proceed to this operator.
        String cCols[] = child.getColumnNames();
        String pCols[] = parent.getColumnNames();
        for (int i = 0; i < cCols.length; i++) {
            boolean found = false;
            //only add the grouping column if it isn't already present
            //  groupby $col1, $col1 ~~ groupby $col1
            for (int j = 0; j < pCols.length; j++) {
                if (pCols[j].equals(cCols[i]))
                    found = true;
            }
            if (!found)
                parent.addColumnName(cCols[i]);
        }
        try {
            attachedTree.setRoot(child.getXATTree().getRoot());
        } catch (Exception e) {
            e.printStackTrace();
        }
    } 
    // else do nothing.
}
/**
 * It used to merge a GB with Agg() on top of NavUnnest/NavCollection into NavCollection.
 * If the GB has Agg() as function, and the GB's context is same as NavUnnest/NavCollection entry.
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param gbOp The GroupBy operator.
 * @param unnestOp The NavUnnest operator.
 * @since  1.0    
 * @see GroupBy
 * @see Navigate
 */
protected Object reduceGroupByNavigate(GroupBy gbOp, Navigate navOp) {

    if (isDebug()) {
        if (navOp instanceof NavUnnest)
            System.out.println("********* In merge GroupBy/ NavUnnest *************");
        else
            if (navOp instanceof NavCollection)
                System.out.println("********* In merge GroupBy/ NavCollection *************");
    }
    boolean mergable = true;
    // Check if the GB has Agg().
    if (!(gbOp.getXATTree().getRoot().getOperator() instanceof Aggregate)) {
        mergable = false;
        if (isDebug())
            System.out.println("Not mergable because Function is not Aggregate.");
    } else
        if (gbOp.getXATTree().getRoot().getOperator().getNode().getChildren().length
            != 0) {
            mergable = false;
            if (isDebug())
                System.out.println(
                    "Not mergable because inner tree of GB is not only aggregate");
        } else
            if (!gbOp
                .getColumnNames()[0]
                .equals(navOp.getEntryPoint()[0].getStepValue())) {
                mergable = false;
                if (isDebug())
                    System.out.println("Not mergable because Bindings do not match.");
            }
    ReWriteResult p1 = new ReWriteResult();
    if (mergable) {
        // update the nav unnest/collection into nav collection.
        NavCollection navCollection = new NavCollectionImp(navOp);
        navOp.getNode().setOperator(navCollection);
        navCollection.setNode(navOp.getNode());

        // delete the parent and done.
        p1.setDeleteParentNode(true);
        // Please notice, here is a potential bug, where if the GB is the root of the subtree.
        // then, it cannot be deleted. Hence, for the sub-tree, we need to create a dummy root.
        // ParserTraverse need to be updated.
    }
    p1.setFinalPosition(true);
    p1.setSwapped(false);

    return p1;
}
/**
 * Recursively splits a Select operator.
 * @return void
 * @param parent The Select operaotr.
 * @since  1.0    
 * @see Select
 */
protected void splitSelect(Select parent) {

    BinExpression be = (BinExpression) parent.getExpression();
    /* 
     * if we have an BinAndExpression here, then we will split this Select in half
     * and call this method on the left and the right halves
     */
    if (be instanceof BinANDExpression) {
        /*
         * we are creating a new XATNode so we have to set the children correctly
         */
        XATNode x1 = new XATNodeImp();
        XATNode parentXATNode = parent.getNode();
        XATNode childXATNode = parentXATNode.getChildren()[0];
        try {
            parentXATNode.setChild(0, x1);
            x1.setParent(parentXATNode);
            x1.setChild(0, childXATNode);
            childXATNode.setParent(x1);

        } catch (XATNodeDoesNotExistException e) {
            //catch this silly exception and do nothing
        }

        //create a new SelectImp with the left expression
        Select newSelect = new SelectImp(x1, be.getLeft());
        x1.setOperator(newSelect);
        //then set the parent's expression to be the right side 
        parent.setExpression(be.getRight());
        //call this method on the left half
        splitSelect(newSelect);
        //then call the method on the right half
        splitSelect(parent);
        return;
    }
    /**
     * handle the or expression
     */
    else
        if (be instanceof BinORExpression) {

        }

    /*
     * else if we have a BinCompExpression,
     * there is no need to alter the Select 
     */
    else
        if (be instanceof BinCOMPExpression) {
            return;
        }
    return;
}
/**
 * Rule to merge Distinct with Distinct
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent Distinct operator.
 * @param child  The child Distinct operator.
 * @since 1.0
 * @see Distinct
 */
public Object visit(Distinct parent, Distinct child) {
    if (isDebug()) {
        System.out.println("********* In merge Distinct / Distinct *************");
    }

    //Get the information from parent and child distinct
    // Begin to searching for merge
    ReWriteResult p1 = new ReWriteResult();

    if (parent.getInputColumnName().equals(child.getInputColumnName())) {

        //delete the parent node, but it will really delete the child b/c we swap them
        if (isDebug()) {
            System.out.println(
                "$$$$$$$$$$$The distinct  child will be deleted $$$$$$$$$$$");
            System.out.println(
                "$$$$$$$$$$$ "
                    + parent.toString()
                    + " is merged with "
                    + child.toString()
                    + " $$$$$$$$$$$");
        }
        p1.setDeleteChildNode(true);
        swap(parent, child);
        p1.setSwapped(true);

        //make sure the parent is in its final position
        p1.setFinalPosition(false);

    } else {
        /*
        * they can be merged so the parent Navigate will be modified
        * the child's output column will be change to that of the parent
        * the child's destination will have the destination of the parent concatted to it
        * then we delete the parent
        */
        p1.setSwapped(false);
        p1.setFinalPosition(true);
    }
    if (isDebug()) {
        System.out.println("********* Finish merge Distinct / Distinct *************");
    }
    return p1;
}
/**
 * Rule to merge Distinct with XATOperator
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent Distinct operator.
 * @param child  The child XATOperator operator.
 * @since 1.0
 * @see Distinct
 * @see XATOperator
 */
public Object visit(Distinct parent, XATOperator child) {
    if (isDebug()) {
        System.out.println("********* In merge Distinct / Distinct *************");
    }

    //Get the information from parent and child distinct
    // Begin to searching for merge
    ReWriteResult p1 = new ReWriteResult();

    p1.setSwapped(false);
    p1.setFinalPosition(true);
    if (isDebug()) {
        System.out.println(
            "********* Finish merge Distinct / XATOperator *************");
    }
    return p1;
}
/**
 * Rule to merge GroupBy with XATOperator
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param gbOp 	 The parent GroupBy operator.
 * @param child  The child XATOperator operator.
 * @since 1.0
 * @see GroupBy
 * @see XATOperator
 */
public Object visit(GroupBy gbOp, XATOperator child) {
    ReWriteResult p1 = new ReWriteResult();

    if (gbOp != null) {
        reduceGroupBy(gbOp);
    }

    if (child instanceof Navigate) {
        return reduceGroupByNavigate(gbOp, (Navigate) child);
    }
    p1.setFinalPosition(true);
    p1.setSwapped(false);

    return p1;
}
/**
 * This is the rule to split a Select in half
 * The Select can be split in half iff it contains an AND or OR
 * If the Select contains an AND then the a new Select is created and the tree looks like this:
 *
 * Before:
 * Select (exp1 AND exp2)
 *
 * After:
 * Select (exp1)
 *   |
 * Select (exp2)
 *
 * If the Select contains an OR then we dont support it yet....
 * Of course the Select can contain an arbitray amount of AND / OR operators so we have
 * deal with that accordingly
 *
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent Select operator.
 * @param child  The child XATOperator operator.
 * @since 1.0
 * @see Select
 */

public Object visit(Select parent, XATOperator child) {
    if (isDebug()) {
        System.out.println("********* In split Select / XATOperator *************");
    }
    ReWriteResult p1 = new ReWriteResult();
    //recursively evaluate how many Select operators we have to make (if any)		
    splitSelect(parent);
    p1.setFinalPosition(true);
    p1.setSwapped(false);

    return p1;
}
/**
 * Rule to merge NavCollection with NavCollection.
 * If the child's output column matches the parents entry point, then
 * the two operators can be merged into one larger NavUnnest iff the output
 * of the child is not referenced anywhere else
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent NavCollection operator.
 * @param child  The child NavCollection operator.
 * @since 1.0
 * @see NavCollection
 */
public Object visit(NavCollection parent, NavCollection child) {
    return mergeNavigates(parent, child);
}
/**
 * Rule to merge NavCollection with NavUnnest
 * These two cannot be merged 
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent NavCollection operator.
 * @param child  The child NavUnnest operator.
 * @since 1.0
 * @see NavCollection
 * @see NavUnnest
 */
 public Object visit(NavCollection parent, NavUnnest child) {
    ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(true);
    return p1;
}
/**
 * Merge NavigateUnnest and a GroupBy together, and make a rename operator
 * out of it.
 *
 * If will check following conditions:
 * . Unnest entry = Groupby's output column.
 * . Groupby only got an aggregate as its function.
 *
 * Then the rename will be from "gb output" to "unnest binding".
 *
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param navOp The parent NavUnnest operator.
 * @param gbOp  The child GroupBy operator.
 * @since 1.0
 * @see GroupBy
 * @see NavUnnest
 */ 
public Object visit(NavUnnest navOp, GroupBy gbOp) {

    if (isDebug()) {
        System.out.println("********* In merge NavUnnest/ Groupby *************");
    }
    boolean mergable = true;
    // Check if the GB has Agg().
    if (!(gbOp.getXATTree().getRoot().getOperator() instanceof Aggregate)) {
        mergable = false;
        if (isDebug())
            System.out.println("Not mergable because Function is not Aggregate.");
    } else
        if (!gbOp
            .getOutputColumnName()
            .equals(navOp.getEntryPoint()[0].getStepValue())) {
            mergable = false;
            if (isDebug())
                System.out.println("Not mergable because Bindings do not match.");
        } else
            if (navOp.getDestination().length > 0) {
                mergable = false;
                if (isDebug())
                    System.out.println("Not mergable because Navigation step is not empty.");
            }
    ReWriteResult p1 = new ReWriteResult();
    if (mergable) {
        // update the NavUnnest + GB into Rename.
        NameColumn rename =
            new NameColumnImp(
                navOp.getNode(),
                gbOp.getOutputColumnName(),
                navOp.getOutputColumnName());
        navOp.getNode().setOperator(rename);
        rename.setNode(navOp.getNode());

        // delete the child and done.
        p1.setDeleteChildNode(true);
    }
    p1.setFinalPosition(true);
    p1.setSwapped(false);

    return p1;

}
/**
 * Rule to merge NavUnnest with NavCollection 
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent NavUnnest operator.
 * @param child  The child NavCollection operator.
 * @since 1.0
 * @see NavCollection
 * @see NavUnnest
 */
public Object visit(NavUnnest parent, NavCollection child) {
    return mergeNavigates(parent, child);
}
/**
 * Rule to merge NavUnnest with NavUnnest
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent NavUnnest operator.
 * @param child  The child NavUnnest operator.
 * @since 1.0
 * @see NavUnnest
 */
public Object visit(NavUnnest parent, NavUnnest child) {
    return mergeNavigates(parent, child);
}
/**
 * Rule to merge Tagger with Merge.
 * This method will iterate through the children of the merge, and merge the child
 * of the merge with the parent if the child is a tagger operator.
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent Tagger operator.
 * @param child  The child Merge operator.
 * @since 1.0
 * @see Tagger
 * @see Merge
 */
public Object visit(Tagger parent, Merge child) {
    if (isDebug()) {
        System.out.println("********* In merge Tagger / Merge *************");
    }

    // get all the children of Merge operator.
    XATNode[] children = child.getNode().getChildren();

    boolean mergeNonDeleteFlag = false;
    // for each child, do the merging.
    for (int i = 0; i < children.length; i++) {
        // get the operator of that children.
        XATOperator xop = children[i].getOperator();
        if (xop instanceof Tagger) {
            this.mergeTaggers(parent, (Tagger) xop);
            // delete the tagger node which is just merged.
            // keep its child if there is any.
            if (children[i].getChildren().length != 0) {
                try {
                    child.getNode().setChild(i, children[i].getChild(0));
                    children[i].getChild(0).setParent(child.getNode());
                    mergeNonDeleteFlag = true;
                } catch (XATNodeDoesNotExistException e) {
                    e.printStackTrace();
                }
            } else {
                child.getNode().deleteChild(children[i]);
            }
        } else
            mergeNonDeleteFlag = true;
    }

    ReWriteResult result = new ReWriteResult();
    result.setFinalPosition(true);
    result.setSwapped(false);

    if (!mergeNonDeleteFlag) {
        result.setDeleteChildNode(true);
    }
    return result;
}
/**
 * Rule to merge Tagger with Tagger 
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent Tagger operator.
 * @param child  The child Tagger operator.
 * @since 1.0
 * @see Tagger
 */
public Object visit(Tagger parent, Tagger child) {
    if (isDebug()) {
        System.out.println("********* In merge Tagger / Tagger *************");
    }

    ReWriteResult p1 = new ReWriteResult();

    if (mergeTaggers(parent, child) > 0) {

        //delete the parent node, but it will really delete the child b/c we swap them
        if (isDebug()) {
            System.out.println("$$$$$$$$$$$The 2 Taggers will be merged$$$$$$$$$$$");
            System.out.println(
                "$$$$$$$$$$$ "
                    + parent.toString()
                    + " is merged with "
                    + child.toString()
                    + " $$$$$$$$$$$");
        }
        p1.setDeleteChildNode(true);
        swap(parent, child);
        p1.setSwapped(true);

        //make sure the parent is in its final position
        p1.setFinalPosition(false);

    } else {
        /*
        * they can be merged so the parent Navigate will be modified
        * the child's output column will be change to that of the parent
        * the child's destination will have the destination of the parent concatted to it
        * then we delete the parent
        */
        p1.setSwapped(false);
        p1.setFinalPosition(true);
    }
    if (isDebug()) {
        System.out.println("********* Finish merge Tagger / Tagger *************");
    }
    return p1;
}
}
